/***
 *  $Id$
 **
 *  File: SequenceIndex.hpp
 *  Created: Sep 16, 2010
 *
 *  Author: Jaroslaw Zola <jaroslaw.zola@gmail.com>
 *          Xiao Yang <eks.yang@gmail.com>
 *
 *  This file is part of CLOSET.
 *
 *  Copyright (c) 2011-2012 Xiao Yang, Jaroslaw Zola, Srinivas Aluru
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use,
 *  copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following
 *  conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef SEQUENCE_INDEX_HPP
#define SEQUENCE_INDEX_HPP

#include <algorithm>
#include <fstream>
#include <string>

#include <boost/tuple/tuple.hpp>


struct SequenceIndex {
    unsigned int id;

    // THIS IS LENGTH OF UNCOMPRESSED SEQUENCE!!!
    unsigned short len;

    // we do not use offset anymore
    // stays for compatibility
    unsigned long off;

}; // struct SequenceIndex


template <typename Iter>
Iter index_find(Iter begin, Iter end, unsigned int id) {
    SequenceIndex key;
    key.id = id;
    Iter first, last;
    boost::tie(first, last) = std::equal_range(begin, end, key);
    if (first == last) return end;
    return first;
} // index_find


inline bool operator<(const SequenceIndex& s1, const SequenceIndex& s2) {
    return s1.id < s2.id;
} // operator<

inline std::ostream& operator<<(std::ostream& os, const SequenceIndex& s) {
    os << s.id << ' ' << s.len << ' ' << s.off;
    return os;
} // operator<<

inline std::istream& operator>>(std::istream& is, SequenceIndex& s) {
    is >> s.id >> s.len >> s.off;
    return is;
} // operator>>

#endif // SEQUENCE_INDEX_HPP
