/***
 *  $Id$
 **
 *  File: SequenceMap.hpp
 *  Created: Mar 30, 2011
 *
 *  Author: Jaroslaw Zola <jaroslaw.zola@gmail.com>
 *
 *  This file is part of CLOSET.
 *
 *  Copyright (c) 2011-2012 Xiao Yang, Jaroslaw Zola, Srinivas Aluru
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use,
 *  copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following
 *  conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef SEQUENCE_MAP_HPP
#define SEQUENCE_MAP_HPP

#include <algorithm>
#include <fstream>
#include <string>
#include <vector>

#include <boost/tuple/tuple.hpp>


struct SequenceObj {
    unsigned int id;
    unsigned int size;
    unsigned int length;
    unsigned int offset;
    // name[0] is the name of representative sequence
    std::vector<std::string> name;
}; // struct SequenceObj

inline bool operator==(const SequenceObj& lhs, const SequenceObj& rhs) {
    return (lhs.id == rhs.id);
} // operator==

inline bool operator<(const SequenceObj& lhs, const SequenceObj& rhs) {
    return (lhs.id < rhs.id);
} // operator<


class SequenceMap {
public:
    typedef std::vector<SequenceObj>::iterator iterator;

    typedef std::vector<SequenceObj>::const_iterator const_iterator;

    iterator begin() { return seqmap_.begin(); }

    const_iterator begin() const { return seqmap_.begin(); }

    iterator end() { return seqmap_.end(); }

    const_iterator end() const { return seqmap_.end(); }

    const_iterator find(unsigned int id) const {
	SequenceObj key;
	key.id = id;

	const_iterator first, last;

	boost::tie(first, last) = std::equal_range(seqmap_.begin(), seqmap_.end(), key);
	if (first == last) return end();

	return first;
    } // find

    bool read(const std::string& name) {
	std::ifstream f(name.c_str());
	if (!f) return false;

	std::string buf;
	SequenceObj sobj;

	while (!f.eof()) {
	    f >> sobj.id;
	    f >> sobj.size;

	    if (!f.eof() && !f) return false;

	    seqmap_.push_back(sobj);

	    std::getline(f, buf);

	    seqmap_.back().name.resize(sobj.size);
	    for (unsigned int i = 0; i < sobj.size; ++i) {
		std::getline(f, seqmap_.back().name[i]);
	    }
	} // while

	f.close();

	std::sort(seqmap_.begin(), seqmap_.end());
	seqmap_.erase(std::unique(seqmap_.begin(), seqmap_.end()), seqmap_.end());

	return true;
    } // read

private:
    std::vector<SequenceObj> seqmap_;

}; // class SequenceMap

#endif // SEQUENCE_MAP_HPP
