/***
 *  $Id$
 **
 *  File: ConfigFile.hpp
 *  Created: Dec 12, 2007
 *
 *  Author: Jaroslaw Zola <jaroslaw.zola@gmail.com>
 *  Copyright (c) 2007-2008 Jaroslaw Zola
 *  Distributed under the Boost Software License.
 *  See accompanying LICENSE.
 *
 *  This file is part of jaz.
 */

#ifndef JAZ_CONFIG_FILE_HPP
#define JAZ_CONFIG_FILE_HPP

#include "string_add.hpp"
#include <fstream>
#include <map>
#include <string>
#include <vector>


namespace jaz {

  class ConfigFile {
  public:

      ConfigFile() : line_(0) { }

      bool read(const std::string& name,
		std::map<std::string, std::string>& cfg) {
	  std::ifstream f(name.c_str());

	  if (!f) return false;

	  line_ = 0;

	  std::vector<std::string> tokens;
	  std::back_insert_iterator<std::vector<std::string> > tokens_ii(tokens);

	  do {
	      std::string s;
	      std::getline(f, s);
	      ++line_;

	      if ((f.good() == false) && (f.eof() == false)) { return false; }

	      if (s.empty() == false) {
		  if (s[0] != '#') {
		      tokens.clear();
		      split('=', s, tokens_ii);
		      if (tokens.size() != 2) return false;
		      else {
			  cfg[remove_space(tokens[0])]
			      = remove_space(tokens[1]);
		      }
		  }
	      }
	  }
	  while (f.eof() == false);

	  f.close();
	  return true;
      } // read

      int error() const { return line_; }

  private:
      int line_;

  }; // class ConfigFile

  inline bool cf_check_str(const std::map<std::string, std::string>& cfg,
			   const std::string& key, std::string& val) {
      std::map<std::string, std::string>::const_iterator iter(cfg.find(key));
      if (iter == cfg.end()) return false;
      val = iter->second;
      return true;
  } // cf_check_str

  template <typename T>
  inline bool cf_check_val(const std::map<std::string, std::string>& cfg,
			   const std::string& key, T& val) {
      std::map<std::string, std::string>::const_iterator iter(cfg.find(key));
      if (iter == cfg.end()) return false;
      string_to st;
      return st(iter->second, val);
  } // cf_check_val

} // namespace jaz

#endif // JAZ_CONFIG_FILE_HPP
