/***
 *  $Id$
 **
 *  File: mr-clique-task2.cpp
 *  Created: Sep 27, 2010
 *
 *  Author: Jaroslaw Zola <jaroslaw.zola@gmail.com>
 *          Xiao Yang <eks.yang@gmail.com>
 *
 *  This file is part of CLOSET.
 *
 *  Copyright (c) 2011-2012 Xiao Yang, Jaroslaw Zola, Srinivas Aluru
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use,
 *  copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following
 *  conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 */

#include <algorithm>
#include <vector>

#include <inttypes.h>

#include <hadoop/Pipes.hh>
#include <hadoop/TemplateFactory.hh>

#include <jaz/string_add.hpp>

#include "mr-common.hpp"


class CliqueMap : public HadoopPipes::Mapper {
public:
    CliqueMap(HadoopPipes::TaskContext& context) { }

    void map(HadoopPipes::MapContext& context) {
	context.emit(context.getInputKey(), context.getInputValue());
    } // map

}; // class CliqueMap


class CliqueReduce : public HadoopPipes::Reducer {
public:
    CliqueReduce(HadoopPipes::TaskContext& context) { }


    void reduce(HadoopPipes::ReduceContext& context) {
	std::vector<std::string> val;
	while (context.nextValue()) val.push_back(context.getInputValue());

	unsigned int l = val.size();
	std::vector<NodeKey> ndkey(l);

	for (unsigned int i = 0; i < l; ++i) {
	    std::vector<std::string> nd;
  	    cluster2nodes(val[i], std::back_inserter(nd));
	    ndkey[i].nodes = jaz::join(' ', nd.begin(), nd.end());
	    ndkey[i].epos = i;
	}

	std::sort(ndkey.begin(), ndkey.end());

	unsigned int pos = 0;

	for (unsigned int i = 1; i < l; ++i) {
	    // we should merge
	    if (ndkey[i].nodes != ndkey[pos].nodes) {
		std::vector<std::string> edges;

		for (unsigned int j = pos; j < i; ++j) {
		    jaz::split(' ', val[ndkey[j].epos], std::back_inserter(edges));
		}

		std::sort(edges.begin(), edges.end());
		std::vector<std::string>::iterator iter(std::unique(edges.begin(), edges.end()));

		std::string S = jaz::join(' ', edges.begin(), iter);

		context.emit(context.getInputKey(), S);

		pos = i;
	    } // if
	} // for i

	// merge remaining data
	std::vector<std::string> edges;

	for (unsigned int j = pos; j < l; ++j) {
	    jaz::split(' ', val[ndkey[j].epos], std::back_inserter(edges));
	}

	std::sort(edges.begin(), edges.end());
	std::vector<std::string>::iterator iter(std::unique(edges.begin(), edges.end()));

	std::string S = jaz::join(' ', edges.begin(), iter);

	context.emit(context.getInputKey(), S);
    } // reduce


private:
    struct NodeKey {
	std::string nodes;
	unsigned int epos;

	bool operator<(const NodeKey& rhs) const { return nodes < rhs.nodes; }

    }; // struct NodeKey

}; // class CliqueReduce


int main(int argc, char* argv[]) {
    return HadoopPipes::runTask(HadoopPipes::TemplateFactory<CliqueMap, CliqueReduce>());
} // main
