/***
 *  $Id$
 **
 *  File: mr-common.hpp
 *  Created: Jul 20, 2011
 *
 *  Author: Jaroslaw Zola <jaroslaw.zola@gmail.com>
 *
 *  This file is part of CLOSET.
 *
 *  Copyright (c) 2011-2012 Xiao Yang, Jaroslaw Zola, Srinivas Aluru
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use,
 *  copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following
 *  conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef MR_COMMON_HPP
#define MR_COMMON_HPP

#include <algorithm>
#include <vector>

#include <hdfs.h>
#include <inttypes.h>

#include <jaz/string_add.hpp>
#include <jaz/utility_add.hpp>


class Partition : public HadoopPipes::Partitioner {
public:
    Partition(HadoopPipes::TaskContext& context) { }

    int partition(const std::string& k, int n) {
	uint64_t v;
	st_(k, v);
	return (v % n);
    } // partition

private:
    jaz::string_to st_;

}; // class Partition


template <typename OutputIter> void cluster2nodes(std::string ed, OutputIter out) {
    std::vector<std::string> nd;
    std::replace(ed.begin(), ed.end(), ' ', ':');
    jaz::split(':', ed, std::back_inserter(nd));
    std::sort(nd.begin(), nd.end());
    std::copy(nd.begin(), std::unique(nd.begin(), nd.end()), out);
} // cluster2nodes


template <typename Iter, typename Hash>
inline uint64_t nodes2hash(Iter first, Iter last, const Hash& h) {
    jaz::string_to st;
    unsigned int n = (last - first);
    jaz::scoped_array<uint64_t> p(new uint64_t[n]);
    const char* c = reinterpret_cast<const char*>(p.get());
    for (unsigned int i = 0; i < n; ++i) st(first[i], p[i]);
    return h(c, n * sizeof(uint64_t));
} // nodes2hash


inline void read_hdfs_block(hdfsFS& fs, hdfsFile& f, char* buf, unsigned long int n) {
    unsigned long int sz = 0;
    unsigned long int pos = 0;
    while (pos < n) {
	sz = hdfsRead(fs, f, buf + pos, n - pos);
	pos += sz;
    }
} // read_hdfs_block

inline bool read_hdfs_file(hdfsFS& fs, const hdfsFileInfo& info, char* buf) {
    hdfsFile rf = hdfsOpenFile(fs, info.mName, O_RDONLY, 0, 0, 0);
    if (rf == 0) return false;
    read_hdfs_block(fs, rf, buf, info.mSize);
    hdfsCloseFile(fs, rf);
    return true;
} // read_hdfs_file

#endif // MR_COMMON_HPP
