/***
 *  $Id$
 **
 *  File: mr-config.cpp
 *  Created: Sep 17, 2010
 *
 *  Author: Jaroslaw Zola <jaroslaw.zola@gmail.com>
 *          Xiao Yang <eks.yang@gmail.com>
 *
 *  This file is part of CLOSET.
 *
 *  Copyright (c) 2011-2012 Xiao Yang, Jaroslaw Zola, Srinivas Aluru
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use,
 *  copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following
 *  conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 */

#include <iostream>
#include <boost/lexical_cast.hpp>
#include <jaz/ConfigFile.hpp>

#include "default.h"


int main(int argc, char* argv[]) {
    if (argc != 2) {
	std::cout << "Usage: " << argv[0] << " <conf>" << std::endl;
	return -1;
    }

    jaz::ConfigFile cf;
    std::map<std::string, std::string> cmap;

    if (cf.read(argv[1], cmap) == false) {
	std::cout << "unable to parse config" << std::endl;
	return -1;
    }

    // we print closet.mvalue to use in closet script
    if (cmap.find("closet.mvalue") == cmap.end()) {
	cmap.insert(std::make_pair("closet.mvalue", boost::lexical_cast<std::string>(CLOSET_MVALUE)));
    }

    if (cmap.find("closet.miter") == cmap.end()) {
	cmap.insert(std::make_pair("closet.miter", boost::lexical_cast<std::string>(CLOSET_MITER)));
    }

    if (cmap.find("closet.sim.init") == cmap.end()) {
	cmap.insert(std::make_pair("closet.sim.init", boost::lexical_cast<std::string>(CLOSET_SIMINIT)));
    }

    if (cmap.find("closet.sim.step") == cmap.end()) {
	cmap.insert(std::make_pair("closet.sim.step", boost::lexical_cast<std::string>(CLOSET_SIMSTEP)));
    }

    if (cmap.find("closet.sim.lim") == cmap.end()) {
	cmap.insert(std::make_pair("closet.sim.lim", boost::lexical_cast<std::string>(CLOSET_SIMLIM)));
    }

    // here we print
    std::map<std::string, std::string>::const_iterator iter(cmap.begin());
    std::cout << "-D " << iter->first << "=" << iter->second;
    ++iter;

    for (; iter != cmap.end(); ++iter) {
	std::cout << " -D " << iter->first << "=" << iter->second;
    }

    return 0;
} // main
