/***
 *  $Id$
 **
 *  File: mr-sketch-task3.cpp
 *  Created: Sep 18, 2010
 *
 *  Author: Jaroslaw Zola <jaroslaw.zola@gmail.com>
 *          Xiao Yang <eks.yang@gmail.com>
 *
 *  This file is part of CLOSET.
 *
 *  Copyright (c) 2011-2012 Xiao Yang, Jaroslaw Zola, Srinivas Aluru
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use,
 *  copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following
 *  conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 */

#include <inttypes.h>

#include <hadoop/Pipes.hh>
#include <hadoop/TemplateFactory.hh>


class SketchMap : public HadoopPipes::Mapper {
public:
    SketchMap(HadoopPipes::TaskContext& context) { }

    void map(HadoopPipes::MapContext& context) {
	context.emit(context.getInputKey(), context.getInputValue());
    } // map

}; // class SketchMap


class SketchReduce : public HadoopPipes::Reducer {
public:
    SketchReduce(HadoopPipes::TaskContext& context) { }

    void reduce(HadoopPipes::ReduceContext& context) {
	std::string k = context.getInputKey();
	std::string seq, S;

	// the idea is to emit the following output:
	// source_seqid source_sequence target_seqids
	// we now that node ids are coded on 5 bytes
	// sequences must be longer than that

	while (context.nextValue()) {
	    seq = context.getInputValue();
	    if (seq.size() == 5) S += seq;
	    else break;
	}

	while (context.nextValue()) S += context.getInputValue();

	if (S.empty() == false)	context.emit(k, (seq + " " + S));
    } // reduce

}; // class SketchReduce


int main(int argc, char* argv[]) {
    // return HadoopPipes::runTask(HadoopPipes::TemplateFactory<SketchMap, SketchReduce, Partition>());
    return HadoopPipes::runTask(HadoopPipes::TemplateFactory<SketchMap, SketchReduce>());
} // main
